@@ -114,15 +114,12 @@ struct _ClutterBoxChild
   ClutterBoxAlignment x_align;
   ClutterBoxAlignment y_align;
 
-  guint x_fill : 1;
-  guint y_fill : 1;
+  ClutterActorBox last_allocation;
 
-  guint expand : 1;
-
-  /* the last stable allocation before an animation; it is
-   * used as the initial ActorBox when interpolating
-   */
-  ClutterActorBox *last_allocation;
+  guint x_fill              : 1;
+  guint y_fill              : 1;
+  guint expand              : 1;
+  guint has_last_allocation : 1;
 };
 
 enum
@@ -360,16 +357,6 @@ clutter_box_child_get_property (GObject    *gobject,
     }
 }
 
-static void
-clutter_box_child_finalize (GObject *gobject)
-{
-  ClutterBoxChild *self = CLUTTER_BOX_CHILD (gobject);
-
-  clutter_actor_box_free (self->last_allocation);
-
-  G_OBJECT_CLASS (clutter_box_child_parent_class)->finalize (gobject);
-}
-
 static void
 clutter_box_child_class_init (ClutterBoxChildClass *klass)
 {
@@ -378,7 +365,6 @@ clutter_box_child_class_init (ClutterBoxChildClass *klass)
 
   gobject_class->set_property = clutter_box_child_set_property;
   gobject_class->get_property = clutter_box_child_get_property;
-  gobject_class->finalize = clutter_box_child_finalize;
 
   pspec = g_param_spec_boolean ("expand",
                                 P_("Expand"),
@@ -434,7 +420,7 @@ clutter_box_child_init (ClutterBoxChild *self)
 
   self->expand = FALSE;
 
-  self->last_allocation = NULL;
+  self->has_last_allocation = FALSE;
 }
 
 static gdouble
@@ -477,9 +463,9 @@ clutter_box_layout_set_container (ClutterLayoutManager *layout,
       /* we need to change the :request-mode of the container
        * to match the orientation
        */
-      request_mode = (priv->is_vertical)
-                   ? CLUTTER_REQUEST_WIDTH_FOR_HEIGHT
-                   : CLUTTER_REQUEST_HEIGHT_FOR_WIDTH;
+      request_mode = (priv->is_vertical
+                      ? CLUTTER_REQUEST_HEIGHT_FOR_WIDTH
+                      : CLUTTER_REQUEST_WIDTH_FOR_HEIGHT);
       clutter_actor_set_request_mode (CLUTTER_ACTOR (priv->container),
                                       request_mode);
     }
@@ -670,6 +656,8 @@ allocate_box_child (ClutterBoxLayout       *self,
       clutter_actor_get_preferred_height (child, avail_width,
                                           NULL, &child_nat);
 
+      child_nat = MIN (child_nat, avail_height);
+
       child_box.y1 = floorf (*position + 0.5);
 
       if (priv->is_homogeneous)
@@ -687,6 +675,8 @@ allocate_box_child (ClutterBoxLayout       *self,
       clutter_actor_get_preferred_width (child, avail_height,
                                          NULL, &child_nat);
 
+      child_nat = MIN (child_nat, avail_width);
+
       child_box.x1 = floorf (*position + 0.5);
 
       if (priv->is_homogeneous)
@@ -715,24 +705,24 @@ allocate_box_child (ClutterBoxLayout       *self,
   if (priv->use_animations && priv->is_animating)
     {
       ClutterLayoutManager *manager = CLUTTER_LAYOUT_MANAGER (self);
-      ClutterActorBox *start = NULL;
-      ClutterActorBox end = { 0, };
+      ClutterActorBox *start, end;
       gdouble p;
 
       p = clutter_layout_manager_get_animation_progress (manager);
 
-      start = box_child->last_allocation;
-      if (start == NULL)
+      if (!box_child->has_last_allocation)
         {
           /* if there is no allocation available then the child has just
            * been added to the container; we put it in the final state
            * and store its allocation for later
            */
-          box_child->last_allocation = clutter_actor_box_copy (&child_box);
+          box_child->last_allocation = child_box;
+          box_child->has_last_allocation = TRUE;
 
           goto do_allocate;
         }
 
+      start = &box_child->last_allocation;
       end = child_box;
 
       /* interpolate between the initial and final values */
@@ -753,7 +743,8 @@ allocate_box_child (ClutterBoxLayout       *self,
   else
     {
       /* store the allocation for later animations */
-      box_child->last_allocation = clutter_actor_box_copy (&child_box);
+      box_child->last_allocation = child_box;
+      box_child->has_last_allocation = TRUE;
     }
 
 do_allocate:
@@ -813,7 +804,7 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
 {
   ClutterBoxLayoutPrivate *priv = CLUTTER_BOX_LAYOUT (layout)->priv;
   gfloat avail_width, avail_height, pref_width, pref_height;
-  gint n_expand_children, extra_space;
+  gint n_expand_children, n_children, extra_space;
   GList *children, *l;
   gfloat position;
   gboolean is_rtl;
@@ -846,7 +837,7 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
     }
 
   /* count the number of children with expand set to TRUE */
-  n_expand_children = 0;
+  n_children = n_expand_children = 0;
   for (l = children; l; l = l->next)
     {
       ClutterLayoutMeta *meta;
@@ -857,21 +848,18 @@ clutter_box_layout_allocate (ClutterLayoutManager   *layout,
 
       if (CLUTTER_BOX_CHILD (meta)->expand)
         n_expand_children++;
+
+      n_children++;
     }
 
   if (priv->is_homogeneous)
     {
-      gint n_children;
-
-      n_children = g_list_length (children);
       if (priv->is_vertical)
-	{
-	  extra_space = (avail_height - (n_children - 1)*priv->spacing)/n_children;
-	}
+	extra_space = (avail_height - (n_children - 1) * priv->spacing)
+                    / n_children;
       else
-	{
-	  extra_space = (avail_width - (n_children - 1)*priv->spacing)/n_children;
-	}
+	extra_space = (avail_width - (n_children - 1) * priv->spacing)
+                    / n_children;
     }
   else if (n_expand_children == 0)
     {
@@ -1087,8 +1075,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    */
   pspec = g_param_spec_boolean ("vertical",
                                 P_("Vertical"),
-                                P_("Whether the layout should be vertical, rather"
-                                   "than horizontal"),
+                                P_("Whether the layout should be vertical, "
+                                   "rather than horizontal"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_VERTICAL, pspec);
@@ -1103,8 +1091,8 @@ clutter_box_layout_class_init (ClutterBoxLayoutClass *klass)
    */
   pspec = g_param_spec_boolean ("homogeneous",
                                 P_("Homogeneous"),
-                                P_("Whether the layout should be homogeneous, i.e."
-                                   "all childs get the same size"),
+                                P_("Whether the layout should be homogeneous, "
+                                   "i.e. all childs get the same size"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_HOMOGENEOUS, pspec);
