@@ -29,7 +29,7 @@
 
 #include "clutter-backend-private.h"
 #include "clutter-debug.h"
-#include "clutter-event.h"
+#include "clutter-event-private.h"
 #include "clutter-keysyms.h"
 #include "clutter-keysyms-table.h"
 #include "clutter-private.h"
@@ -47,6 +47,7 @@
 typedef struct _ClutterEventPrivate {
   ClutterEvent base;
 
+  ClutterInputDevice *device;
   ClutterInputDevice *source_device;
 
   gpointer platform_data;
@@ -494,38 +495,13 @@ clutter_event_get_device_id (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
-  g_return_val_if_fail (event != NULL, -1);
-
-  switch (event->type)
-    {
-    case CLUTTER_NOTHING:
-    case CLUTTER_STAGE_STATE:
-    case CLUTTER_DESTROY_NOTIFY:
-    case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
-    case CLUTTER_ENTER:
-    case CLUTTER_LEAVE:
-      break;
-    case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_BUTTON_RELEASE:
-      device = event->button.device;
-      break;
-    case CLUTTER_MOTION:
-      device = event->motion.device;
-      break;
-    case CLUTTER_SCROLL:
-      device = event->scroll.device;
-      break;
-    case CLUTTER_KEY_PRESS:
-    case CLUTTER_KEY_RELEASE:
-      device = event->scroll.device;
-      break;
-    }
+  g_return_val_if_fail (event != NULL, CLUTTER_POINTER_DEVICE);
 
+  device = clutter_event_get_device (event);
   if (device != NULL)
     return clutter_input_device_get_device_id (device);
-  else
-    return -1;
+
+  return -1;
 }
 
 /**
@@ -546,6 +522,36 @@ clutter_event_get_device_type (const ClutterEvent *event)
 
   g_return_val_if_fail (event != NULL, CLUTTER_POINTER_DEVICE);
 
+  device = clutter_event_get_device (event);
+  if (device != NULL)
+    return clutter_input_device_get_device_type (device);
+
+  return CLUTTER_POINTER_DEVICE;
+}
+
+/**
+ * clutter_event_set_device:
+ * @event: a #ClutterEvent
+ * @device: a #ClutterInputDevice
+ *
+ * Sets the device for @event.
+ *
+ * Since: 1.6
+ */
+void
+clutter_event_set_device (ClutterEvent       *event,
+                          ClutterInputDevice *device)
+{
+  g_return_if_fail (event != NULL);
+  g_return_if_fail (device == NULL || CLUTTER_IS_INPUT_DEVICE (device));
+
+  if (is_event_allocated (event))
+    {
+      ClutterEventPrivate *real_event = (ClutterEventPrivate *) event;
+
+      real_event->device = device;
+    }
+
   switch (event->type)
     {
     case CLUTTER_NOTHING:
@@ -559,27 +565,22 @@ clutter_event_get_device_type (const ClutterEvent *event)
 
     case CLUTTER_BUTTON_PRESS:
     case CLUTTER_BUTTON_RELEASE:
-      device = event->button.device;
+      event->button.device = device;
       break;
 
     case CLUTTER_MOTION:
-      device = event->motion.device;
+      event->motion.device = device;
       break;
 
     case CLUTTER_SCROLL:
-      device = event->scroll.device;
+      event->scroll.device = device;
       break;
 
     case CLUTTER_KEY_PRESS:
     case CLUTTER_KEY_RELEASE:
-      device = event->scroll.device;
+      event->key.device = device;
       break;
     }
-
-  if (device != NULL)
-    return clutter_input_device_get_device_type (device);
-  else
-    return CLUTTER_POINTER_DEVICE;
 }
 
 /**
@@ -604,6 +605,14 @@ clutter_event_get_device (const ClutterEvent *event)
 
   g_return_val_if_fail (event != NULL, NULL);
 
+  if (is_event_allocated (event))
+    {
+      ClutterEventPrivate *real_event = (ClutterEventPrivate *) event;
+
+      if (real_event->device != NULL)
+        return real_event->device;
+    }
+
   switch (event->type)
     {
     case CLUTTER_NOTHING:
@@ -691,6 +700,7 @@ clutter_event_copy (const ClutterEvent *event)
     {
       ClutterEventPrivate *real_event = (ClutterEventPrivate *) event;
 
+      new_real_event->device = real_event->device;
       new_real_event->source_device = real_event->source_device;
     }
 
@@ -783,7 +793,7 @@ clutter_event_get (void)
 {
   ClutterMainContext *context = _clutter_context_get_default ();
 
-  if (!context->events_queue)
+  if (context->events_queue == NULL)
     return NULL;
 
   if (g_queue_is_empty (context->events_queue))
@@ -825,9 +835,11 @@ _clutter_event_push (const ClutterEvent *event,
   ClutterMainContext *context = _clutter_context_get_default ();
   ClutterInputDevice *device;
 
-  /* FIXME: check queue is valid */
   g_assert (context != NULL);
 
+  if (context->events_queue == NULL)
+    context->events_queue = g_queue_new ();
+
   /* disabled devices don't propagate events */
   device = clutter_event_get_device (event);
   if (device != NULL)
@@ -883,7 +895,7 @@ clutter_events_pending (void)
 
   g_return_val_if_fail (context != NULL, FALSE);
 
-  if (!context->events_queue)
+  if (context->events_queue == NULL)
     return FALSE;
 
   return g_queue_is_empty (context->events_queue) == FALSE;
@@ -1002,6 +1014,17 @@ _clutter_event_set_device (ClutterEvent       *event,
     }
 }
 
+/*< private >
+ * clutter_event_set_source_device:
+ * @event: a #ClutterEvent
+ * @device: a #ClutterInputDevice
+ *
+ * Sets the source #ClutterInputDevice for @event.
+ *
+ * The #ClutterEvent must have been created using clutter_event_new().
+ *
+ * Since: 1.6
+ */
 void
 _clutter_event_set_source_device (ClutterEvent       *event,
                                   ClutterInputDevice *device)
