@@ -104,6 +104,8 @@ struct _ClutterFlowLayoutPrivate
   /* per-line size */
   GArray *line_min;
   GArray *line_natural;
+  gfloat req_width;
+  gfloat req_height;
 
   guint line_count;
 
@@ -124,9 +126,13 @@ enum
   PROP_MIN_COLUMN_WIDTH,
   PROP_MAX_COLUMN_WIDTH,
   PROP_MIN_ROW_HEGHT,
-  PROP_MAX_ROW_HEIGHT
+  PROP_MAX_ROW_HEIGHT,
+
+  N_PROPERTIES
 };
 
+static GParamSpec *flow_properties[N_PROPERTIES] = { NULL, };
+
 G_DEFINE_TYPE (ClutterFlowLayout,
                clutter_flow_layout,
                CLUTTER_TYPE_LAYOUT_MANAGER);
@@ -356,6 +362,8 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
                 total_natural_width,
                 for_height);
 
+  priv->req_height = for_height;
+
   if (min_width_p)
     *min_width_p = total_min_width;
 
@@ -531,6 +539,8 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
                 total_natural_height,
                 for_width);
 
+  priv->req_width = for_width;
+
   if (min_height_p)
     *min_height_p = total_min_height;
 
@@ -557,6 +567,21 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
 
   clutter_actor_box_get_size (allocation, &avail_width, &avail_height);
 
+  /* blow the cached preferred size and re-compute with the given
+   * available size in case the FlowLayout wasn't given the exact
+   * size it requested
+   */
+  if ((priv->req_width > 0 && avail_width != priv->req_width) ||
+      (priv->req_height > 0 && avail_height != priv->req_height))
+    {
+      clutter_flow_layout_get_preferred_width (manager, container,
+                                               avail_height,
+                                               NULL, NULL);
+      clutter_flow_layout_get_preferred_height (manager, container,
+                                                avail_width,
+                                                NULL, NULL);
+    }
+
   items_per_line = compute_lines (CLUTTER_FLOW_LAYOUT (manager),
                                   avail_width, avail_height);
 
@@ -829,17 +854,12 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
 {
   GObjectClass *gobject_class;
   ClutterLayoutManagerClass *layout_class;
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterFlowLayoutPrivate));
 
   gobject_class = G_OBJECT_CLASS (klass);
   layout_class = CLUTTER_LAYOUT_MANAGER_CLASS (klass);
 
-  gobject_class->set_property = clutter_flow_layout_set_property;
-  gobject_class->get_property = clutter_flow_layout_get_property;
-  gobject_class->finalize = clutter_flow_layout_finalize;
-
   layout_class->get_preferred_width =
     clutter_flow_layout_get_preferred_width;
   layout_class->get_preferred_height =
@@ -857,14 +877,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_enum ("orientation",
-                             P_("Orientation"),
-                             P_("The orientation of the layout"),
-                             CLUTTER_TYPE_FLOW_ORIENTATION,
-                             CLUTTER_FLOW_HORIZONTAL,
-                             CLUTTER_PARAM_READWRITE |
-                             G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class, PROP_ORIENTATION, pspec);
+  flow_properties[PROP_ORIENTATION] =
+    g_param_spec_enum ("orientation",
+                       P_("Orientation"),
+                       P_("The orientation of the layout"),
+                       CLUTTER_TYPE_FLOW_ORIENTATION,
+                       CLUTTER_FLOW_HORIZONTAL,
+                       CLUTTER_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
   /**
    * ClutterFlowLayout:homogeneous:
@@ -874,12 +893,12 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_boolean ("homogeneous",
-                                P_("Homogeneous"),
-                                P_("Whether each item should receive the same allocation"),
-                                FALSE,
-                                CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_HOMOGENEOUS, pspec);
+  flow_properties[PROP_HOMOGENEOUS] =
+    g_param_spec_boolean ("homogeneous",
+                          P_("Homogeneous"),
+                          P_("Whether each item should receive the same allocation"),
+                          FALSE,
+                          CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:column-spacing:
@@ -890,15 +909,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("column-spacing",
-                              P_("Column Spacing"),
-                              P_("The spacing between columns"),
-                              0.0, G_MAXFLOAT,
-                              0.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_COLUMN_SPACING,
-                                   pspec);
+  flow_properties[PROP_COLUMN_SPACING] =
+    g_param_spec_float ("column-spacing",
+                        P_("Column Spacing"),
+                        P_("The spacing between columns"),
+                        0.0, G_MAXFLOAT,
+                        0.0,
+                        CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:row-spacing:
@@ -909,15 +926,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("row-spacing",
-                              P_("Row Spacing"),
-                              P_("The spacing between rows"),
-                              0.0, G_MAXFLOAT,
-                              0.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_ROW_SPACING,
-                                   pspec);
+  flow_properties[PROP_ROW_SPACING] =
+    g_param_spec_float ("row-spacing",
+                        P_("Row Spacing"),
+                        P_("The spacing between rows"),
+                        0.0, G_MAXFLOAT,
+                        0.0,
+                        CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:min-column-width:
@@ -926,15 +941,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("min-column-width",
-                              P_("Minimum Column Width"),
-                              P_("Minimum width for each column"),
-                              0.0, G_MAXFLOAT,
-                              0.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_MIN_COLUMN_WIDTH,
-                                   pspec);
+  flow_properties[PROP_MIN_COLUMN_WIDTH] =
+    g_param_spec_float ("min-column-width",
+                        P_("Minimum Column Width"),
+                        P_("Minimum width for each column"),
+                        0.0, G_MAXFLOAT,
+                        0.0,
+                        CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:max-column-width:
@@ -944,15 +957,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("max-column-width",
-                              P_("Maximum Column Width"),
-                              P_("Maximum width for each column"),
-                              -1.0, G_MAXFLOAT,
-                              -1.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_MAX_COLUMN_WIDTH,
-                                   pspec);
+  flow_properties[PROP_MAX_COLUMN_WIDTH] =
+    g_param_spec_float ("max-column-width",
+                        P_("Maximum Column Width"),
+                        P_("Maximum width for each column"),
+                        -1.0, G_MAXFLOAT,
+                        -1.0,
+                        CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:min-row-height:
@@ -961,15 +972,13 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("min-row-height",
-                              P_("Minimum Row Height"),
-                              P_("Minimum height for each row"),
-                              0.0, G_MAXFLOAT,
-                              0.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_MIN_ROW_HEGHT,
-                                   pspec);
+  flow_properties[PROP_MIN_ROW_HEGHT] =
+    g_param_spec_float ("min-row-height",
+                        P_("Minimum Row Height"),
+                        P_("Minimum height for each row"),
+                        0.0, G_MAXFLOAT,
+                        0.0,
+                        CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterFlowLayout:max-row-height:
@@ -979,15 +988,20 @@ clutter_flow_layout_class_init (ClutterFlowLayoutClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_float ("max-row-height",
-                              P_("Maximum Row Height"),
-                              P_("Maximum height for each row"),
-                              -1.0, G_MAXFLOAT,
-                              -1.0,
-                              CLUTTER_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_MAX_ROW_HEIGHT,
-                                   pspec);
+  flow_properties[PROP_MAX_ROW_HEIGHT] =
+    g_param_spec_float ("max-row-height",
+                        P_("Maximum Row Height"),
+                        P_("Maximum height for each row"),
+                        -1.0, G_MAXFLOAT,
+                        -1.0,
+                        CLUTTER_PARAM_READWRITE);
+
+  gobject_class->finalize = clutter_flow_layout_finalize;
+  gobject_class->set_property = clutter_flow_layout_set_property;
+  gobject_class->get_property = clutter_flow_layout_get_property;
+  _clutter_object_class_install_properties (gobject_class,
+                                            N_PROPERTIES,
+                                            flow_properties);
 }
 
 static void
@@ -1073,7 +1087,8 @@ clutter_flow_layout_set_orientation (ClutterFlowLayout      *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (layout), "orientation");
+      _clutter_notify_by_pspec (G_OBJECT (layout),
+                                flow_properties[PROP_ORIENTATION]);
     }
 }
 
@@ -1125,7 +1140,8 @@ clutter_flow_layout_set_homogeneous (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (layout), "homogeneous");
+      _clutter_notify_by_pspec (G_OBJECT (layout),
+                                flow_properties[PROP_HOMOGENEOUS]);
     }
 }
 
@@ -1175,7 +1191,8 @@ clutter_flow_layout_set_column_spacing (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (layout), "column-spacing");
+      _clutter_notify_by_pspec (G_OBJECT (layout),
+                                flow_properties[PROP_COLUMN_SPACING]);
     }
 }
 
@@ -1226,7 +1243,8 @@ clutter_flow_layout_set_row_spacing (ClutterFlowLayout *layout,
       manager = CLUTTER_LAYOUT_MANAGER (layout);
       clutter_layout_manager_layout_changed (manager);
 
-      g_object_notify (G_OBJECT (layout), "row-spacing");
+      _clutter_notify_by_pspec (G_OBJECT (layout),
+                                flow_properties[PROP_ROW_SPACING]);
     }
 }
 
@@ -1295,10 +1313,12 @@ clutter_flow_layout_set_column_width (ClutterFlowLayout *layout,
     }
 
   if (notify_min)
-    g_object_notify (G_OBJECT (layout), "min-column-width");
+    _clutter_notify_by_pspec (G_OBJECT (layout),
+                              flow_properties[PROP_MIN_COLUMN_WIDTH]);
 
   if (notify_max)
-    g_object_notify (G_OBJECT (layout), "max-column-width");
+    _clutter_notify_by_pspec (G_OBJECT (layout),
+                              flow_properties[PROP_MAX_COLUMN_WIDTH]);
 
   g_object_thaw_notify (G_OBJECT (layout));
 }
@@ -1373,10 +1393,12 @@ clutter_flow_layout_set_row_height (ClutterFlowLayout *layout,
     }
 
   if (notify_min)
-    g_object_notify (G_OBJECT (layout), "min-row-height");
+    _clutter_notify_by_pspec (G_OBJECT (layout),
+                              flow_properties[PROP_MIN_ROW_HEGHT]);
 
   if (notify_max)
-    g_object_notify (G_OBJECT (layout), "max-row-height");
+    _clutter_notify_by_pspec (G_OBJECT (layout),
+                              flow_properties[PROP_MAX_ROW_HEIGHT]);
 
   g_object_thaw_notify (G_OBJECT (layout));
 }
