@@ -109,6 +109,7 @@
 #include "clutter-master-clock.h"
 #include "clutter-private.h"
 #include "clutter-profile.h"
+#include "clutter-stage-manager.h"
 #include "clutter-stage-private.h"
 #include "clutter-version.h" 	/* For flavour define */
 
@@ -227,54 +228,6 @@ clutter_get_accessibility_enabled (void)
   return cally_get_cally_initialized ();
 }
 
-void
-_clutter_do_redraw (ClutterStage *stage)
-{
-  static GTimer *timer = NULL;
-  static guint timer_n_frames = 0;
-  ClutterMainContext *ctx;
-
-  ctx = _clutter_context_get_default ();
-
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
-  _clutter_stage_reset_picks_per_frame_counter (stage);
-
-  _clutter_backend_ensure_context (ctx->backend, stage);
-
-  /* Setup FPS count - not currently across *all* stages rather than per */
-  if (G_UNLIKELY (clutter_get_show_fps ()))
-    {
-      if (!timer)
-	timer = g_timer_new ();
-    }
-
-  /* The code below can't go in stage paint as base actor_paint
-   * will get called before it (and break picking, etc)
-   */
-  _clutter_stage_maybe_setup_viewport (stage);
-
-  /* Call through to the actual backend to do the painting down from
-   * the stage. It will likely need to swap buffers, vblank sync etc
-   * which will be windowing system dependent
-  */
-  _clutter_backend_redraw (ctx->backend, stage);
-
-  /* Complete FPS info */
-  if (G_UNLIKELY (clutter_get_show_fps ()))
-    {
-      timer_n_frames++;
-
-      if (g_timer_elapsed (timer, NULL) >= 1.0)
-	{
-	  g_print ("*** FPS: %i ***\n", timer_n_frames);
-	  timer_n_frames = 0;
-	  g_timer_start (timer);
-	}
-    }
-
-  CLUTTER_TIMESTAMP (SCHEDULER, "Redraw finish for stage:%p", stage);
-}
-
 /**
  * clutter_redraw:
  *
@@ -312,9 +265,28 @@ clutter_redraw (ClutterStage *stage)
 void
 clutter_set_motion_events_enabled (gboolean enable)
 {
-  ClutterMainContext *context = _clutter_context_get_default ();
+  ClutterStageManager *stage_manager;
+  ClutterMainContext *context;
+  const GSList *l;
 
+  enable = !!enable;
+
+  context = _clutter_context_get_default ();
+  if (context->motion_events_per_actor == enable)
+    return;
+
+  /* store the flag for later query and for newly created stages */
   context->motion_events_per_actor = enable;
+
+  /* propagate the change to all stages */
+  stage_manager = clutter_stage_manager_get_default ();
+
+  for (l = clutter_stage_manager_peek_stages (stage_manager);
+       l != NULL;
+       l = l->next)
+    {
+      _clutter_stage_set_motion_events_enabled (l->data, enable);
+    }
 }
 
 /**
@@ -335,17 +307,18 @@ clutter_get_motion_events_enabled (void)
 }
 
 static inline ClutterActor *
-_clutter_actor_get_by_id (guint32 id)
+_clutter_actor_get_by_id (guint32 actor_id)
 {
   ClutterMainContext *context = _clutter_context_get_default ();
 
   g_assert (context->id_pool != NULL);
 
-  return clutter_id_pool_lookup (context->id_pool, id);
+  return _clutter_id_pool_lookup (context->id_pool, actor_id);
 }
 
 void
-_clutter_id_to_color (guint id, ClutterColor *col)
+_clutter_id_to_color (guint         id_,
+                      ClutterColor *col)
 {
   ClutterMainContext *ctx;
   gint red, green, blue;
@@ -373,11 +346,11 @@ _clutter_id_to_color (guint id, ClutterColor *col)
     }
 
   /* compute the numbers we'll store in the components */
-  red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
+  red   = (id_ >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
         & (0xff >> (8-ctx->fb_r_mask_used));
-  green = (id >> ctx->fb_b_mask_used)
+  green = (id_ >> ctx->fb_b_mask_used)
         & (0xff >> (8-ctx->fb_g_mask_used));
-  blue  = (id)
+  blue  = (id_)
         & (0xff >> (8-ctx->fb_b_mask_used));
 
   /* shift left bits a bit and add one, this circumvents
@@ -414,12 +387,12 @@ _clutter_id_to_color (guint id, ClutterColor *col)
     }
 }
 
-guint
+static inline guint
 _clutter_pixel_to_id (guchar pixel[4])
 {
   ClutterMainContext *ctx;
-  gint  red, green, blue;
-  guint id;
+  gint red, green, blue;
+  guint retval;
 
   ctx = _clutter_context_get_default ();
 
@@ -455,16 +428,17 @@ _clutter_pixel_to_id (guchar pixel[4])
   blue  = blue  >> (ctx->fb_b_mask - ctx->fb_b_mask_used);
 
   /* combine the correct per component values into the final id */
-  id = blue
-     + (green <<  ctx->fb_b_mask_used)
-     + (red << (ctx->fb_b_mask_used + ctx->fb_g_mask_used));
+  retval = blue
+         + (green <<  ctx->fb_b_mask_used)
+         + (red << (ctx->fb_b_mask_used + ctx->fb_g_mask_used));
 
-  return id;
+  return retval;
 }
 
 #ifdef USE_GDKPIXBUF
 static void
-pixbuf_free (guchar *pixels, gpointer data)
+pixbuf_free (guchar   *pixels,
+             gpointer  data)
 {
   g_free (pixels);
 }
@@ -472,10 +446,10 @@ pixbuf_free (guchar *pixels, gpointer data)
 
 static void
 read_pixels_to_file (char *filename_stem,
-                     int x,
-                     int y,
-                     int width,
-                     int height)
+                     int   x,
+                     int   y,
+                     int   width,
+                     int   height)
 {
 #ifdef USE_GDKPIXBUF
   GLubyte *data;
@@ -535,12 +509,12 @@ _clutter_do_pick (ClutterStage   *stage,
 		  ClutterPickMode mode)
 {
   ClutterMainContext *context;
-  guchar              pixel[4] = { 0xff, 0xff, 0xff, 0xff };
-  CoglColor           stage_pick_id;
-  guint32             id;
-  GLboolean           dither_was_on;
-  ClutterActor       *actor;
-  gboolean            is_clipped;
+  guchar pixel[4] = { 0xff, 0xff, 0xff, 0xff };
+  CoglColor stage_pick_id;
+  guint32 id_;
+  GLboolean dither_was_on;
+  ClutterActor *actor;
+  gboolean is_clipped;
   CLUTTER_STATIC_COUNTER (do_pick_counter,
                           "_clutter_do_pick counter",
                           "Increments for each full pick run",
@@ -606,8 +580,8 @@ _clutter_do_pick (ClutterStage   *stage,
           goto result;
         }
 
-      id = _clutter_pixel_to_id (pixel);
-      actor = _clutter_actor_get_by_id (id);
+      id_ = _clutter_pixel_to_id (pixel);
+      actor = _clutter_actor_get_by_id (id_);
       goto result;
     }
 
@@ -698,8 +672,8 @@ _clutter_do_pick (ClutterStage   *stage,
       goto result;
     }
 
-  id = _clutter_pixel_to_id (pixel);
-  actor = _clutter_actor_get_by_id (id);
+  id_ = _clutter_pixel_to_id (pixel);
+  actor = _clutter_actor_get_by_id (id_);
 
 result:
 
@@ -713,13 +687,15 @@ result:
   return actor;
 }
 
-CoglPangoFontMap *
-_clutter_context_get_pango_fontmap (ClutterMainContext *self)
+static CoglPangoFontMap *
+clutter_context_get_pango_fontmap (void)
 {
+  ClutterMainContext *self;
   CoglPangoFontMap *font_map;
   gdouble resolution;
   gboolean use_mipmapping;
 
+  self = _clutter_context_get_default ();
   if (G_LIKELY (self->font_map != NULL))
     return self->font_map;
 
@@ -812,13 +788,15 @@ update_pango_context (ClutterBackend *backend,
 }
 
 PangoContext *
-_clutter_context_get_pango_context (ClutterMainContext *self)
+_clutter_context_get_pango_context (void)
 {
+  ClutterMainContext *self = _clutter_context_get_default ();
+
   if (G_UNLIKELY (self->pango_context == NULL))
     {
       PangoContext *context;
 
-      context = _clutter_context_create_pango_context (self);
+      context = _clutter_context_create_pango_context ();
       self->pango_context = context;
 
       g_signal_connect (self->backend, "resolution-changed",
@@ -835,15 +813,15 @@ _clutter_context_get_pango_context (ClutterMainContext *self)
 }
 
 PangoContext *
-_clutter_context_create_pango_context (ClutterMainContext *self)
+_clutter_context_create_pango_context (void)
 {
   CoglPangoFontMap *font_map;
   PangoContext *context;
 
-  font_map = _clutter_context_get_pango_fontmap (self);
+  font_map = clutter_context_get_pango_fontmap ();
 
   context = cogl_pango_font_map_create_context (font_map);
-  update_pango_context (self->backend, context);
+  update_pango_context (clutter_get_default_backend (), context);
   pango_context_set_language (context, pango_language_get_default ());
 
   return context;
@@ -915,12 +893,9 @@ clutter_main (void)
   if (clutter_main_loop_level == 0)
     CLUTTER_TIMER_START (uprof_get_mainloop_context (), mainloop_timer);
 
-  /* Make sure there is a context */
-  CLUTTER_CONTEXT ();
-
-  if (!clutter_is_initialized)
+  if (!_clutter_context_is_initialized ())
     {
-      g_warning ("Called clutter_main() but Clutter wasn't initialised.  "
+      g_warning ("Called clutter_main() but Clutter wasn't initialised. "
 		 "You must call clutter_init() first.");
       return;
     }
@@ -1616,7 +1591,7 @@ pre_parse_hook (GOptionContext  *context,
 
   clutter_context = _clutter_context_get_default ();
 
-  clutter_context->id_pool = clutter_id_pool_new (256);
+  clutter_context->id_pool = _clutter_id_pool_new (256);
 
   backend = clutter_context->backend;
   g_assert (CLUTTER_IS_BACKEND (backend));
@@ -2052,12 +2027,10 @@ event_click_count_generate (ClutterEvent *event)
 
   ClutterInputDevice *device = NULL;
   ClutterSettings *settings;
-  ClutterBackend *backend;
   guint double_click_time;
   guint double_click_distance;
 
   settings = clutter_settings_get_default ();
-  backend = clutter_get_default_backend ();
 
   g_object_get (settings,
                 "double-click-distance", &double_click_distance,
@@ -2400,10 +2373,8 @@ _clutter_process_event_details (ClutterActor        *stage,
         break;
 
       case CLUTTER_MOTION:
-        /* Only stage gets motion events if clutter_set_motion_events is TRUE,
-         * and the event is not a synthetic event with source set.
-         */
-        if (!context->motion_events_per_actor &&
+        /* only the stage gets motion events if they are enabled */
+        if (!_clutter_stage_get_motion_events_enabled (CLUTTER_STAGE (stage)) &&
             event->any.source == NULL)
           {
             /* Only stage gets motion events */
@@ -2413,13 +2384,15 @@ _clutter_process_event_details (ClutterActor        *stage,
             if (context->pointer_grab_actor != NULL)
               {
                 clutter_actor_event (context->pointer_grab_actor,
-                                     event, FALSE);
+                                     event,
+                                     FALSE);
                 break;
               }
             else if (device != NULL && device->pointer_grab_actor != NULL)
               {
                 clutter_actor_event (device->pointer_grab_actor,
-                                     event, FALSE);
+                                     event,
+                                     FALSE);
                 break;
               }
 
@@ -2565,9 +2538,9 @@ _clutter_process_event (ClutterEvent *event)
 
 /**
  * clutter_get_actor_by_gid:
- * @id: a #ClutterActor ID.
+ * @id_: a #ClutterActor unique id.
  *
- * Retrieves the #ClutterActor with @id.
+ * Retrieves the #ClutterActor with @id_.
  *
  * Return value: (transfer none): the actor with the passed id or %NULL.
  *   The returned actor does not have its reference count increased.
@@ -2575,9 +2548,9 @@ _clutter_process_event (ClutterEvent *event)
  * Since: 0.6
  */
 ClutterActor *
-clutter_get_actor_by_gid (guint32 id)
+clutter_get_actor_by_gid (guint32 id_)
 {
-  return _clutter_actor_get_by_id (id);
+  return _clutter_actor_get_by_id (id_);
 }
 
 void
@@ -2713,7 +2686,7 @@ clutter_grab_pointer (ClutterActor *actor)
 /**
  * clutter_grab_pointer_for_device:
  * @actor: a #ClutterActor
- * @id: a device id, or -1
+ * @id_: a device id, or -1
  *
  * Grabs all the pointer events coming from the device @id for @actor.
  *
@@ -2723,22 +2696,21 @@ clutter_grab_pointer (ClutterActor *actor)
  */
 void
 clutter_grab_pointer_for_device (ClutterActor *actor,
-                                 gint          id)
+                                 gint          id_)
 {
   ClutterInputDevice *dev;
 
   g_return_if_fail (actor == NULL || CLUTTER_IS_ACTOR (actor));
 
   /* essentially a global grab */
-  if (id == -1)
+  if (id_ == -1)
     {
       clutter_grab_pointer (actor);
       return;
     }
 
-  dev = clutter_get_input_device_for_id (id);
-
-  if (!dev)
+  dev = clutter_get_input_device_for_id (id_);
+  if (dev == NULL)
     return;
 
   if (dev->pointer_grab_actor == actor)
@@ -2747,8 +2719,8 @@ clutter_grab_pointer_for_device (ClutterActor *actor,
   if (dev->pointer_grab_actor)
     {
       g_object_weak_unref (G_OBJECT (dev->pointer_grab_actor),
-                          on_pointer_grab_weak_notify,
-                          dev);
+                           on_pointer_grab_weak_notify,
+                           dev);
       dev->pointer_grab_actor = NULL;
     }
 
@@ -2757,8 +2729,8 @@ clutter_grab_pointer_for_device (ClutterActor *actor,
       dev->pointer_grab_actor = actor;
 
       g_object_weak_ref (G_OBJECT (actor),
-                        on_pointer_grab_weak_notify,
-                        dev);
+                         on_pointer_grab_weak_notify,
+                         dev);
     }
 }
 
@@ -2778,16 +2750,16 @@ clutter_ungrab_pointer (void)
 
 /**
  * clutter_ungrab_pointer_for_device:
- * @id: a device id
+ * @id_: a device id
  *
- * Removes an existing grab of the pointer events for device @id.
+ * Removes an existing grab of the pointer events for device @id_.
  *
  * Since: 0.8
  */
 void
-clutter_ungrab_pointer_for_device (gint id)
+clutter_ungrab_pointer_for_device (gint id_)
 {
-  clutter_grab_pointer_for_device (NULL, id);
+  clutter_grab_pointer_for_device (NULL, id_);
 }
 
 
@@ -2917,7 +2889,7 @@ clutter_clear_glyph_cache (void)
 {
   CoglPangoFontMap *font_map;
 
-  font_map = _clutter_context_get_pango_fontmap (CLUTTER_CONTEXT ());
+  font_map = clutter_context_get_pango_fontmap ();
   cogl_pango_font_map_clear_glyph_cache (font_map);
 }
 
@@ -2949,7 +2921,7 @@ clutter_set_font_flags (ClutterFontFlags flags)
 
   backend = clutter_get_default_backend ();
 
-  font_map = _clutter_context_get_pango_fontmap (context);
+  font_map = clutter_context_get_pango_fontmap ();
   use_mipmapping = (flags & CLUTTER_FONT_MIPMAPPING) != 0;
   cogl_pango_font_map_set_use_mipmapping (font_map, use_mipmapping);
 
@@ -2990,21 +2962,21 @@ clutter_set_font_flags (ClutterFontFlags flags)
 ClutterFontFlags
 clutter_get_font_flags (void)
 {
-  ClutterMainContext *context = CLUTTER_CONTEXT ();
   CoglPangoFontMap *font_map = NULL;
   const cairo_font_options_t *font_options;
   ClutterFontFlags flags = 0;
+  cairo_hint_style_t hint_style;
 
-  font_map = _clutter_context_get_pango_fontmap (context);
+  font_map = clutter_context_get_pango_fontmap ();
   if (cogl_pango_font_map_get_use_mipmapping (font_map))
     flags |= CLUTTER_FONT_MIPMAPPING;
 
-  font_options = clutter_backend_get_font_options (context->backend);
+  font_options =
+    clutter_backend_get_font_options (clutter_get_default_backend ());
 
-  if ((cairo_font_options_get_hint_style (font_options)
-       != CAIRO_HINT_STYLE_DEFAULT)
-      && (cairo_font_options_get_hint_style (font_options)
-          != CAIRO_HINT_STYLE_NONE))
+  hint_style = cairo_font_options_get_hint_style (font_options);
+  if (hint_style != CAIRO_HINT_STYLE_DEFAULT &&
+      hint_style != CAIRO_HINT_STYLE_NONE)
     flags |= CLUTTER_FONT_HINTING;
 
   return flags;
@@ -3012,9 +2984,9 @@ clutter_get_font_flags (void)
 
 /**
  * clutter_get_input_device_for_id:
- * @id: the unique id for a device
+ * @id_: the unique id for a device
  *
- * Retrieves the #ClutterInputDevice from its @id. This is a convenience
+ * Retrieves the #ClutterInputDevice from its @id_. This is a convenience
  * wrapper for clutter_device_manager_get_device() and it is functionally
  * equivalent to:
  *
@@ -3031,13 +3003,13 @@ clutter_get_font_flags (void)
  * Since: 0.8
  */
 ClutterInputDevice *
-clutter_get_input_device_for_id (gint id)
+clutter_get_input_device_for_id (gint id_)
 {
   ClutterDeviceManager *manager;
 
   manager = clutter_device_manager_get_default ();
 
-  return clutter_device_manager_get_device (manager, id);
+  return clutter_device_manager_get_device (manager, id_);
 }
 
 /**
@@ -3055,9 +3027,7 @@ clutter_get_input_device_for_id (gint id)
 PangoFontMap *
 clutter_get_font_map (void)
 {
-  ClutterMainContext *context = _clutter_context_get_default ();
-
-  return PANGO_FONT_MAP (_clutter_context_get_pango_fontmap (context));
+  return PANGO_FONT_MAP (clutter_context_get_pango_fontmap ());
 }
 
 typedef struct _ClutterRepaintFunction
@@ -3085,7 +3055,7 @@ clutter_threads_remove_repaint_func (guint handle_id)
 
   g_return_if_fail (handle_id > 0);
 
-  context = CLUTTER_CONTEXT ();
+  context = _clutter_context_get_default ();
   l = context->repaint_funcs;
   while (l != NULL)
     {
@@ -3149,7 +3119,7 @@ clutter_threads_add_repaint_func (GSourceFunc    func,
 
   g_return_val_if_fail (func != NULL, 0);
 
-  context = CLUTTER_CONTEXT ();
+  context = _clutter_context_get_default ();
 
   /* XXX lock the context */
 
@@ -3180,7 +3150,7 @@ clutter_threads_add_repaint_func (GSourceFunc    func,
 void
 _clutter_run_repaint_functions (void)
 {
-  ClutterMainContext *context = CLUTTER_CONTEXT ();
+  ClutterMainContext *context = _clutter_context_get_default ();
   ClutterRepaintFunction *repaint_func;
   GList *reinvoke_list, *l;
 
@@ -3289,6 +3259,22 @@ _clutter_clear_events_queue (void)
     }
 }
 
+guint32
+_clutter_context_acquire_id (gpointer key)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  return _clutter_id_pool_add (context->id_pool, key);
+}
+
+void
+_clutter_context_release_id (guint32 id_)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  return _clutter_id_pool_remove (context->id_pool, id_);
+}
+
 void
 _clutter_clear_events_queue_for_stage (ClutterStage *stage)
 {
@@ -3312,3 +3298,40 @@ _clutter_clear_events_queue_for_stage (ClutterStage *stage)
         }
     }
 }
+
+ClutterPickMode
+_clutter_context_get_pick_mode (void)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  return context->pick_mode;
+}
+
+void
+_clutter_context_push_shader_stack (ClutterActor *actor)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  context->shaders = g_slist_prepend (context->shaders, actor);
+}
+
+ClutterActor *
+_clutter_context_peek_shader_stack (void)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  if (context->shaders != NULL)
+    return context->shaders->data;
+
+  return NULL;
+}
+
+ClutterActor *
+_clutter_context_pop_shader_stack (ClutterActor *actor)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  context->shaders = g_slist_remove (context->shaders, actor);
+
+  return _clutter_context_peek_shader_stack ();
+}
