@@ -252,9 +252,16 @@ _cogl_pipeline_texture_storage_change_notify (CoglHandle texture)
        * we continue to check the rest */
     }
 }
+
 static void
 set_glsl_program (GLuint gl_program)
 {
+#ifdef HAVE_COGL_GLES
+
+  g_return_if_reached ();
+
+#else /* HAVE_COGL_GLES */
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (ctx->current_gl_program != gl_program)
@@ -272,6 +279,8 @@ set_glsl_program (GLuint gl_program)
           ctx->current_gl_program = 0;
         }
     }
+
+#endif /* HAVE_COGL_GLES */
 }
 
 void
@@ -556,9 +565,10 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
                          blend_state->blend_dst_factor_rgb));
     }
 
+#ifndef HAVE_COGL_GLES2
+
   /* Under GLES2 the alpha function is implemented as part of the
      fragment shader */
-#ifndef HAVE_COGL_GLES2
   if (pipelines_difference & (COGL_PIPELINE_STATE_ALPHA_FUNC |
                               COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE))
     {
@@ -571,6 +581,28 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
       GE (glAlphaFunc (alpha_state->alpha_func,
                        alpha_state->alpha_func_reference));
     }
+
+  /* Under GLES2 the lighting parameters are implemented as uniforms
+     in the progend */
+  if (pipelines_difference & COGL_PIPELINE_STATE_LIGHTING)
+    {
+      CoglPipeline *authority =
+        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LIGHTING);
+      CoglPipelineLightingState *lighting_state =
+        &authority->big_state->lighting_state;
+
+      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,
+                        lighting_state->ambient));
+      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,
+                        lighting_state->diffuse));
+      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,
+                        lighting_state->specular));
+      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,
+                        lighting_state->emission));
+      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SHININESS,
+                        &lighting_state->shininess));
+    }
+
 #endif /* HAVE_COGL_GLES2 */
 
   if (pipelines_difference & COGL_PIPELINE_STATE_DEPTH)
@@ -674,11 +706,11 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
       return FALSE;
     }
 
-  if (layers_difference & COGL_PIPELINE_LAYER_STATE_TEXTURE)
+  if (layers_difference & COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA)
     {
+      unsigned long state = COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA;
       CoglPipelineLayer *authority =
-        _cogl_pipeline_layer_get_authority (layer,
-                                            COGL_PIPELINE_LAYER_STATE_TEXTURE);
+        _cogl_pipeline_layer_get_authority (layer, state);
       CoglHandle texture;
       GLuint     gl_texture;
       GLenum     gl_target;
@@ -909,7 +941,8 @@ compare_layer_differences_cb (CoglPipelineLayer *layer, void *user_data)
    * then we force an update of the texture state...
    */
   if (unit->texture_storage_changed)
-    state->layer_differences[state->i] |= COGL_PIPELINE_LAYER_STATE_TEXTURE;
+    state->layer_differences[state->i] |=
+      COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA;
 
   state->i++;
 
