@@ -37,19 +37,20 @@
 #include <GL/gl.h>
 
 #include "clutter-backend-glx.h"
-#include "clutter-event-glx.h"
 #include "clutter-stage-glx.h"
 #include "clutter-glx.h"
 #include "clutter-profile.h"
 
-#include "../clutter-event.h"
-#include "../clutter-main.h"
-#include "../clutter-debug.h"
-#include "../clutter-private.h"
-#include "../clutter-version.h"
+#include "clutter-debug.h"
+#include "clutter-event-translator.h"
+#include "clutter-event.h"
+#include "clutter-main.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 #include "cogl/cogl.h"
 
+#define clutter_backend_glx_get_type    _clutter_backend_glx_get_type
 
 G_DEFINE_TYPE (ClutterBackendGLX, clutter_backend_glx, CLUTTER_TYPE_BACKEND_X11);
 
@@ -59,7 +60,7 @@ static ClutterBackendGLX *backend_singleton = NULL;
 static gchar    *clutter_vblank_name = NULL;
 
 G_CONST_RETURN gchar*
-clutter_backend_glx_get_vblank_method (void)
+_clutter_backend_glx_get_vblank_method (void)
 {
   return clutter_vblank_name;
 }
@@ -68,6 +69,8 @@ static gboolean
 clutter_backend_glx_pre_parse (ClutterBackend  *backend,
                                GError         **error)
 {
+  ClutterBackendClass *parent_class =
+    CLUTTER_BACKEND_CLASS (clutter_backend_glx_parent_class);
   const gchar *env_string;
 
   env_string = g_getenv ("CLUTTER_VBLANK");
@@ -77,7 +80,7 @@ clutter_backend_glx_pre_parse (ClutterBackend  *backend,
       env_string = NULL;
     }
 
-  return clutter_backend_x11_pre_parse (backend, error);
+  return parent_class->pre_parse (backend, error);
 }
 
 static gboolean
@@ -86,11 +89,11 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
-  ClutterBackendClass *backend_class =
+  ClutterBackendClass *parent_class =
     CLUTTER_BACKEND_CLASS (clutter_backend_glx_parent_class);
   int glx_major, glx_minor;
 
-  if (!backend_class->post_parse (backend, error))
+  if (!parent_class->post_parse (backend, error))
     return FALSE;
 
   if (!glXQueryExtension (backend_x11->xdpy,
@@ -107,8 +110,8 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
   /* XXX: Technically we should require >= GLX 1.3 support but for a long
    * time Mesa has exported a hybrid GLX, exporting extensions specified
    * to require GLX 1.3, but still reporting 1.2 via glXQueryVersion. */
-  if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor)
-      || !(glx_major == 1 && glx_minor >= 2))
+  if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor) ||
+      !(glx_major == 1 && glx_minor >= 2))
     {
       g_set_error (error, CLUTTER_INIT_ERROR,
                    CLUTTER_INIT_ERROR_BACKEND,
@@ -133,8 +136,12 @@ static void
 clutter_backend_glx_add_options (ClutterBackend *backend,
                                  GOptionGroup   *group)
 {
+  ClutterBackendClass *parent_class =
+    CLUTTER_BACKEND_CLASS (clutter_backend_glx_parent_class);
+
   g_option_group_add_entries (group, entries);
-  clutter_backend_x11_add_options (backend, group);
+
+  parent_class->add_options (backend, group);
 }
 
 static void
@@ -159,7 +166,7 @@ clutter_backend_glx_dispose (GObject *gobject)
     {
       glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
       glXDestroyContext (backend_x11->xdpy, backend_glx->gl_context);
-      backend_glx->gl_context = None;
+      backend_glx->gl_context = NULL;
     }
 
   if (backend_glx->dummy_glxwin)
@@ -214,15 +221,19 @@ static ClutterFeatureFlags
 clutter_backend_glx_get_features (ClutterBackend *backend)
 {
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
+  ClutterBackendClass *parent_class;
   const gchar *glx_extensions = NULL;
+  const gchar *gl_extensions = NULL;
   ClutterFeatureFlags flags;
   gboolean use_dri = FALSE;
 
-  flags = clutter_backend_x11_get_features (backend);
+  parent_class = CLUTTER_BACKEND_CLASS (clutter_backend_glx_parent_class);
+
+  flags = parent_class->get_features (backend);
   flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
 
   /* this will make sure that the GL context exists */
-  g_assert (backend_glx->gl_context != None);
+  g_assert (backend_glx->gl_context != NULL);
   g_assert (glXGetCurrentDrawable () != None);
 
   CLUTTER_NOTE (BACKEND,
@@ -242,6 +253,26 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
 
   CLUTTER_NOTE (BACKEND, "  GLX Extensions: %s", glx_extensions);
 
+  gl_extensions = (const gchar *)glGetString (GL_EXTENSIONS);
+
+  /* When using glBlitFramebuffer or glXCopySubBufferMESA for sub stage
+   * redraws, we cannot rely on glXSwapIntervalSGI to throttle the blits
+   * so we need to resort to manually synchronizing with the vblank so we
+   * always check for the video_sync extension...
+   */
+  if (_cogl_check_extension ("GLX_SGI_video_sync", glx_extensions) &&
+      /* Note: the GLX_SGI_video_sync spec explicitly states this extension
+       * only works for direct contexts. */
+      glXIsDirect (clutter_x11_get_default_display (),
+                   backend_glx->gl_context))
+    {
+      backend_glx->get_video_sync =
+        (GetVideoSyncProc) cogl_get_proc_address ("glXGetVideoSyncSGI");
+
+      backend_glx->wait_video_sync =
+        (WaitVideoSyncProc) cogl_get_proc_address ("glXWaitVideoSyncSGI");
+    }
+
   use_dri = check_vblank_env ("dri");
 
   /* First check for explicit disabling or it set elsewhere (eg NVIDIA) */
@@ -312,12 +343,6 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
     {
       CLUTTER_NOTE (BACKEND, "attempting glXGetVideoSyncSGI vblank setup");
 
-      backend_glx->get_video_sync =
-        (GetVideoSyncProc) cogl_get_proc_address ("glXGetVideoSyncSGI");
-
-      backend_glx->wait_video_sync =
-        (WaitVideoSyncProc) cogl_get_proc_address ("glXWaitVideoSyncSGI");
-
       if ((backend_glx->get_video_sync != NULL) &&
           (backend_glx->wait_video_sync != NULL))
         {
@@ -363,6 +388,17 @@ vblank_setup_done:
     {
       backend_glx->copy_sub_buffer =
         (CopySubBufferProc) cogl_get_proc_address ("glXCopySubBufferMESA");
+      backend_glx->can_blit_sub_buffer = TRUE;
+      backend_glx->blit_sub_buffer_is_synchronized = TRUE;
+    }
+  else if (_cogl_check_extension ("GL_EXT_framebuffer_blit", gl_extensions))
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Using glBlitFramebuffer fallback for sub_buffer copies");
+      backend_glx->blit_framebuffer =
+        (BlitFramebufferProc) cogl_get_proc_address ("glBlitFramebuffer");
+      backend_glx->can_blit_sub_buffer = TRUE;
+      backend_glx->blit_sub_buffer_is_synchronized = FALSE;
     }
 
   CLUTTER_NOTE (BACKEND, "backend features checked");
@@ -394,7 +430,7 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
     None
   };
 
-  if (backend_x11->xdpy == None || backend_x11->xscreen == None)
+  if (backend_x11->xdpy == NULL || backend_x11->xscreen == NULL)
     return FALSE;
 
   /* If we don't already have a cached config then try to get one */
@@ -420,7 +456,7 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
 
                   vinfo = glXGetVisualFromFBConfig (backend_x11->xdpy,
                                                     configs[i]);
-                  if (vinfo == None)
+                  if (vinfo == NULL)
                     continue;
 
                   if (vinfo->depth == 32 &&
@@ -429,7 +465,8 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
                        vinfo->blue_mask  == 0x0000ff))
                     {
                       CLUTTER_NOTE (BACKEND,
-                                    "Found GLX visual ARGB [index:%d]", i);
+                                    "Found an ARGB FBConfig [index:%d]",
+                                    i);
 
                       backend_glx->found_fbconfig = TRUE;
                       backend_glx->fbconfig = configs[i];
@@ -445,6 +482,7 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
 
           if (n_configs >= 1)
             {
+              CLUTTER_NOTE (BACKEND, "Using the first available FBConfig");
               backend_glx->found_fbconfig = TRUE;
               backend_glx->fbconfig = configs[0];
             }
@@ -454,14 +492,46 @@ _clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
         }
     }
 
-  if (backend_glx->found_fbconfig)
+  if (G_LIKELY (backend_glx->found_fbconfig))
     {
       *config = backend_glx->fbconfig;
 
       return TRUE;
     }
-  else
-    return FALSE;
+
+  return FALSE;
+}
+
+void
+_clutter_backend_glx_blit_sub_buffer (ClutterBackendGLX *backend_glx,
+                                      GLXDrawable        drawable,
+                                      int                x,
+                                      int                y,
+                                      int                width,
+                                      int                height)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend_glx);
+
+  if (backend_glx->copy_sub_buffer)
+    {
+      backend_glx->copy_sub_buffer (backend_x11->xdpy, drawable,
+                                    x, y, width, height);
+    }
+  else if (backend_glx->blit_framebuffer)
+    {
+      glDrawBuffer (GL_FRONT);
+      backend_glx->blit_framebuffer (x, y, x + width, y + height,
+                                     x, y, x + width, y + height,
+                                     GL_COLOR_BUFFER_BIT, GL_NEAREST);
+      glDrawBuffer (GL_BACK);
+    }
+
+  /* NB: unlike glXSwapBuffers, glXCopySubBuffer and
+   * glBlitFramebuffer don't issue an implicit glFlush() so we
+   * have to flush ourselves if we want the request to complete in
+   * finite amount of time since otherwise the driver can batch
+   * the command indefinitely. */
+  glFlush();
 }
 
 static XVisualInfo *
@@ -492,7 +562,7 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
   int minor;
   GLXDrawable dummy_drawable;
 
-  if (backend_glx->gl_context != None)
+  if (backend_glx->gl_context != NULL)
     return TRUE;
 
   xdisplay = clutter_x11_get_default_display ();
@@ -513,7 +583,7 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
                                                  GLX_RGBA_TYPE,
                                                  NULL,
                                                  True);
-  if (backend_glx->gl_context == None)
+  if (backend_glx->gl_context == NULL)
     {
       g_set_error (error, CLUTTER_INIT_ERROR,
                    CLUTTER_INIT_ERROR_BACKEND,
@@ -540,7 +610,7 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
    * now this is the best solution available.
    */
   xvisinfo = glXGetVisualFromFBConfig (xdisplay, config);
-  if (xvisinfo == None)
+  if (xvisinfo == NULL)
     {
       g_set_error (error, CLUTTER_INIT_ERROR,
                    CLUTTER_INIT_ERROR_BACKEND,
@@ -616,7 +686,7 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
    * implementation attached to it then we clear the GL context
    */
   if (stage == NULL ||
-      (CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_IN_DESTRUCTION) ||
+      CLUTTER_ACTOR_IN_DESTRUCTION (stage) ||
       ((impl = _clutter_stage_get_window (stage)) == NULL))
     {
       ClutterBackendX11 *backend_x11;
@@ -649,7 +719,7 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
                     (unsigned int) drawable);
 
       /* no GL context to set */
-      if (backend_glx->gl_context == None)
+      if (backend_glx->gl_context == NULL)
         return;
 
       clutter_x11_trap_x_errors ();
@@ -707,45 +777,25 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
     }
 }
 
-/*
- * FIXME: we should remove backend_class->redraw() and just
- * have stage_window_iface->redraw()
- */
-static void
-clutter_backend_glx_redraw (ClutterBackend *backend,
-                            ClutterStage   *stage)
-{
-  ClutterStageWindow *impl = _clutter_stage_get_window (stage);
-
-  if (G_UNLIKELY (impl == NULL))
-    {
-      CLUTTER_NOTE (BACKEND, "Stage [%p] has no implementation", stage);
-      return;
-    }
-
-  g_assert (CLUTTER_IS_STAGE_GLX (impl));
-
-  clutter_stage_glx_redraw (CLUTTER_STAGE_GLX (impl),
-                            stage);
-}
-
 static ClutterStageWindow *
 clutter_backend_glx_create_stage (ClutterBackend  *backend,
                                   ClutterStage    *wrapper,
                                   GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterEventTranslator *translator;
   ClutterStageWindow *stage_window;
   ClutterStageX11 *stage_x11;
 
-  CLUTTER_NOTE (BACKEND, "Creating stage of type '%s'",
-                g_type_name (CLUTTER_STAGE_TYPE));
-
   stage_window = g_object_new (CLUTTER_TYPE_STAGE_GLX, NULL);
 
   /* copy backend data into the stage */
   stage_x11 = CLUTTER_STAGE_X11 (stage_window);
   stage_x11->wrapper = wrapper;
+  stage_x11->backend = backend_x11;
+
+  translator = CLUTTER_EVENT_TRANSLATOR (stage_x11);
+  _clutter_backend_add_event_translator (backend, translator);
 
   CLUTTER_NOTE (BACKEND,
                 "GLX stage created[%p] (dpy:%p, screen:%d, root:%u, wrap:%p)",
@@ -774,23 +824,20 @@ clutter_backend_glx_class_init (ClutterBackendGLXClass *klass)
   backend_class->create_stage   = clutter_backend_glx_create_stage;
   backend_class->add_options    = clutter_backend_glx_add_options;
   backend_class->get_features   = clutter_backend_glx_get_features;
-  backend_class->redraw         = clutter_backend_glx_redraw;
   backend_class->create_context = clutter_backend_glx_create_context;
   backend_class->ensure_context = clutter_backend_glx_ensure_context;
 
   backendx11_class->get_visual_info = clutter_backend_glx_get_visual_info;
-  backendx11_class->handle_event = clutter_backend_glx_handle_event;
 }
 
 static void
 clutter_backend_glx_init (ClutterBackendGLX *backend_glx)
 {
-
 }
 
 /* every backend must implement this function */
 GType
 _clutter_backend_impl_get_type (void)
 {
-  return clutter_backend_glx_get_type ();
+  return _clutter_backend_glx_get_type ();
 }
